/*
 *  Copyright © 2016, 2017, Oracle and/or its affiliates. All rights reserved.
 *  This software is dual-licensed to you under the MIT License (MIT) and
 *  the Universal Permissive License (UPL). See the LICENSE file in the root
 *  directory for license terms. You may choose either license, or both.
 */

import Foundation

/**
 * This class represents the response from the server after a request
 * to receive the activation policy.
 */
class ActivationPolicyResponse {
    
    private var keyType: String
    private var keySize: Int
    private var hashAlgorithm: String
    
    init(keyType: String, keySize: Int, hashAlgorithm: String) {
        self.keyType = keyType
        self.keySize = keySize
        self.hashAlgorithm = hashAlgorithm
    }
    
    func getKeyType() -> String {
        return keyType
    }
    
    func getKeySize() -> Int {
        return keySize
    }
    
    func getHashAlgorithm() -> String {
        return hashAlgorithm
    }
    
    func toString() -> String {
        return "ActivationPolicyResponse{" +
                "keyType='" + keyType + "'" +
                ", keySize=" + keySize.description +
                ", hashAlgorithm='" + hashAlgorithm + "'" + "}"
    }
    
    private static let FIELD_KEY_TYPE: String  = "keyType"
    private static let FIELD_HASH_ALGORITHM: String = "hashAlgorithm"
    private static let FIELD_KEY_SIZE: String  = "keySize"
    
    private static func parseJsonStr(str: String) throws -> [String : AnyObject]? {
        if let data = str.data(using: .utf8) {
            return try JSONSerialization.jsonObject(with: data, options: []) as? [String : AnyObject]
        }
        return nil
    }
    
    static func fromJson(data: Foundation.Data?) throws -> ActivationPolicyResponse? {
        if data == nil {
            return nil
        }
        let jsonStr: String = String(data: data!, encoding: .utf8)!
        let jsonDict: [String: AnyObject]? = try parseJsonStr(str: jsonStr)
        if jsonDict == nil {
            return nil
        }
        return ActivationPolicyResponse(keyType: jsonDict![FIELD_KEY_TYPE] as! String,
                                        keySize: jsonDict![FIELD_KEY_SIZE] as! Int,
                                        hashAlgorithm: jsonDict![FIELD_HASH_ALGORITHM] as! String)
    }
    
    func toJsonString() -> String {
        var resultStr: String = "{"
        resultStr = resultStr + "\"" + ActivationPolicyResponse.FIELD_KEY_TYPE + "\":\"" + keyType + "\","
        resultStr = resultStr + "\"" + ActivationPolicyResponse.FIELD_KEY_SIZE + "\":" + keySize.description  + ","
        resultStr = resultStr + "\"" + ActivationPolicyResponse.FIELD_HASH_ALGORITHM + "\":\"" + hashAlgorithm + "\"}"
        return resultStr
    }
}

