/*
 *  Copyright © 2016, 2017, Oracle and/or its affiliates. All rights reserved.
 *  This software is dual-licensed to you under the MIT License (MIT) and
 *  the Universal Permissive License (UPL). See the LICENSE file in the root
 *  directory for license terms. You may choose either license, or both.
 */

import Foundation
import SwiftyJSON

class CertificationRequestInfo {
    static let FIELD_SUBJECT: String = "subject"
    static let FIELD_SUBJECT_PUBLIC_KEY_INFO: String = "subjectPublicKeyInfo"
    static let FIELD_ATTRIBUTES: String = "attributes"
    private var subject: String? = ""
    private var subjectPublicKeyInfo: SubjectPublicKeyInfo?
    private var attributes: [String : AnyObject]? = [ : ]
    
    func getSubject() -> String {
        return subject!
    }
    
    func setSubject(subject: String) {
        self.subject = subject
    }
    
    func getAttributes() -> [String : AnyObject] {
        return attributes!
    }
    
    func setAttributes(attributes: [String : AnyObject]?) {
        self.attributes = attributes;
    }
    
    
    func getSubjectPublicKeyInfo() -> SubjectPublicKeyInfo {
        return subjectPublicKeyInfo!
    }
    
    func setSubjectPublicKeyInfo(subjectPublicKeyInfo: SubjectPublicKeyInfo) {
        self.subjectPublicKeyInfo = subjectPublicKeyInfo;
    }
    
    func toString() -> String {
        var result: String = "CertificationRequestInfo{"
        result = result + "subject='" + subject! + "'"
        result = result + ", subjectPublicKeyInfo=" + subjectPublicKeyInfo!.toString()
        result = result + ", attributes=" + "}"
        return result
    }
    
    func toJsonString() throws -> String {
        var resultStr: String = ""
        resultStr = resultStr + "{"
        resultStr = resultStr + "\"" + CertificationRequestInfo.FIELD_SUBJECT + "\":\""
            + subject! + "\","
        try resultStr = resultStr + "\"" + CertificationRequestInfo.FIELD_SUBJECT_PUBLIC_KEY_INFO +
            "\":" + subjectPublicKeyInfo!.toJsonString() + ","
        resultStr = resultStr + "\"" + CertificationRequestInfo.FIELD_ATTRIBUTES + "\":{"
        let attributeNumber: Int = (attributes! as NSDictionary).count
        var z: Int = 0
        if attributes != nil {
            for (key, val) in attributes! {
                // TODO: what if attribute value is not a String?
                resultStr = resultStr + "\"" + key + "\":" + val.description
                z += 1
                if z < attributeNumber {
                    resultStr = resultStr + ","
                }
            }
        }
        resultStr = resultStr + "}}"
        return resultStr
    }
    
    
    func fromJson(jsonDict: [String : JSON]?) throws {
        if jsonDict == nil {
            return
        }
        setSubject(subject: jsonDict![CertificationRequestInfo.FIELD_SUBJECT]!.string!)
        subjectPublicKeyInfo = SubjectPublicKeyInfo()
        let subjectPublicKeyInfoObj: [String:JSON]? =
            jsonDict![CertificationRequestInfo.FIELD_SUBJECT_PUBLIC_KEY_INFO]?.dictionaryValue
        if (subjectPublicKeyInfoObj != nil) {
            try subjectPublicKeyInfo!.fromJson(jsonDict: subjectPublicKeyInfoObj!)
        }
        let fieldAttributesValue: JSON? = jsonDict![CertificationRequestInfo.FIELD_ATTRIBUTES]
        if(fieldAttributesValue != nil) {
            setAttributes(attributes: CertificationRequestInfo.getJsonMap(value: fieldAttributesValue, name: CertificationRequestInfo.FIELD_ATTRIBUTES))
        }
    }
    
    // TODO: taken from server JsonUtils, refactor!
    static func getJsonMap(value: JSON?, name: String) -> [String : AnyObject]? {
        if value == nil {
            return nil
        }
        let jsonDict = value?.dictionaryValue
        var items = jsonDict!["items"]?.dictionaryValue
        if items == nil {
            if jsonDict!["links"] == nil {
                items = jsonDict
            } else {
                return Dictionary<String, AnyObject>()
            }
        }
        var result: [String : AnyObject] = Dictionary<String, AnyObject>()
        for (key, jo) in items! {
            let v = jo
            result[key] = v as AnyObject?
        }
        return result;
    }
    
}

/**
 * DirectActivationRequest
 */
class DirectActivationRequest {
    
    static let FIELD_CERTIFICATION_REQUEST_INFO: String = "certificationRequestInfo"
    static let FIELD_SIGNATURE_ALGORITHM: String = "signatureAlgorithm"
    static let FIELD_SIGNATURE: String = "signature"
    static let FIELD_DEVICE_MODELS: String = "deviceModels"
    
    private var deviceModels: [String]? = []
    
    private var certificationRequestInfo: CertificationRequestInfo?
    private var signatureAlgorithm: String? = ""
    private var signature: Foundation.Data? = Foundation.Data()
    
    func getCertificationRequestInfo() -> CertificationRequestInfo {
        return certificationRequestInfo!
    }
    
    func setCertificationRequestInfo(certificationRequestInfo: CertificationRequestInfo ) {
        self.certificationRequestInfo = certificationRequestInfo
    }
    
    func getSignatureAlgorithm() -> String {
        return signatureAlgorithm!
    }
    
    func setSignatureAlgorithm(signatureAlgorithm: String) {
        self.signatureAlgorithm = signatureAlgorithm
    }
    
    func getSignature() -> Foundation.Data {
        return signature!
    }
    
    func setSignature(signature: Foundation.Data) {
        self.signature = signature
    }
    
    func getDeviceModels() -> [String] {
        return self.deviceModels!
    }
    
    func addDeviceModel(deviceModel: String) {
        if deviceModels == nil {
            deviceModels = []
        }
        deviceModels!.append(deviceModel)
    }
    
    private static func parseJsonStr(str: String) throws -> [String : AnyObject]? {
        if let data = str.data(using: .utf8) {
            return try JSONSerialization.jsonObject(with: data, options: []) as? [String : AnyObject]
        }
        return nil
    }
    
    static func fromJson(jsonStr: String) throws -> DirectActivationRequest? {
        let result: DirectActivationRequest = DirectActivationRequest()
        let _ = try fromJson(jsonStr: jsonStr, dar: result)
        return result
    }
    
    static func fromJson(jsonStr: String, dar: DirectActivationRequest) throws -> DirectActivationRequest? {
        let data:Foundation.Data? = jsonStr.data(using: .utf8)!
        let jsonDict = try JSON(data: data!)
        let certificationRequestInfo: CertificationRequestInfo = CertificationRequestInfo()
        
        let fieldJsonObject: [String:JSON]? = jsonDict[DirectActivationRequest.FIELD_CERTIFICATION_REQUEST_INFO].dictionaryValue
        if (fieldJsonObject != nil) {
            try certificationRequestInfo.fromJson(jsonDict: fieldJsonObject!)
        }
        dar.setSignatureAlgorithm(signatureAlgorithm: jsonDict[DirectActivationRequest.FIELD_SIGNATURE_ALGORITHM].string!)
        
        let encodedSignature: String = jsonDict[DirectActivationRequest.FIELD_SIGNATURE].string!
        let signatureData: Foundation.Data =
            Foundation.Data(base64Encoded: encodedSignature,
                options: Foundation.Data.Base64DecodingOptions(rawValue: 0))!
        dar.setSignature(signature: signatureData)
        let deviceModelsArray: [JSON]? = jsonDict[DirectActivationRequest.FIELD_DEVICE_MODELS].array
        if deviceModelsArray != nil {
            for deviceModel in deviceModelsArray! {
                dar.deviceModels!.append(deviceModel.string!)
            }
        }
        return dar
    }
    
    func fromJson(data: Foundation.Data?) throws {
        if data == nil {
            return
        }
        let jsonStr: String = String(data: data!, encoding: .utf8)!
        let _ = try DirectActivationRequest.fromJson(jsonStr: jsonStr, dar: self)
    }
    
    func toJsonString() throws -> String? {
        var jsonStr: String = ""
        let encodedSignature: String = signature!.base64EncodedString(
            options: Foundation.Data.Base64EncodingOptions(rawValue: 0))
        jsonStr = jsonStr + "{"
        jsonStr = jsonStr + "\"" + DirectActivationRequest.FIELD_SIGNATURE_ALGORITHM +
            "\":\"" + signatureAlgorithm! + "\","
        try jsonStr = jsonStr + "\"" + DirectActivationRequest.FIELD_CERTIFICATION_REQUEST_INFO + "\":" + certificationRequestInfo!.toJsonString() + ","
        jsonStr = jsonStr + "\"" + DirectActivationRequest.FIELD_SIGNATURE + "\":\"" +
            encodedSignature + "\","
        jsonStr = jsonStr + "\"" + DirectActivationRequest.FIELD_DEVICE_MODELS + "\":["
        var z: Int = 0
        for dm in deviceModels! {
            let characters = Array(dm)
            if characters[0] != "{" {
                jsonStr = jsonStr + "\"" + dm + "\""
            }
            else {
                jsonStr = jsonStr + dm
            }
            z += 1
            if z < deviceModels!.count {
                jsonStr = jsonStr + ","
            }
        }
        jsonStr = jsonStr + "]}"
        return jsonStr
    }
    
    func toString() -> String {
        var result: String = "DirectActivationRequest{"
        result = result + "deviceModels=["
        var i: Int = 0
        for dm in deviceModels! {
            result = result + dm
            if i < deviceModels!.count - 1 {
                result = result + ", "
            }
            i += 1
        }
        result = result + "]"
        result = result + ", certificationRequestInfo=" + certificationRequestInfo!.toString()
        result = result + ", signatureAlgorithm='" + signatureAlgorithm! + "'"
        result = result + ", signature=" + (signature?.description)! + "}"
        return result
    }
}
