/*
 *  Copyright © 2016 Oracle and/or its affiliates. All rights reserved.
 *  This software is dual-licensed to you under the MIT License (MIT) and
 *  the Universal Permissive License (UPL). See the LICENSE file in the root
 *  directory for license terms. You may choose either license, or both.
 */

import Foundation

/**
 * DeviceModelImpl
 */
class DeviceModelImpl: DeviceModel, Hashable {
    
    var urn: String
    var name: String
    var description: String
    var deviceModelAttributes: [String : DeviceModelAttribute] = [ : ]
    var deviceModelActions: [String : DeviceModelAction] = [ : ]
    var deviceModelFormats: [String : DeviceModelFormat] = [ : ]

    convenience init(urn: String, name: String, description: String,
                     deviceModelAttributes: [DeviceModelAttribute]?,
                     deviceModelActions: [DeviceModelAction]?) {
        self.init(urn: urn, name: name, description: description, deviceModelAttributes: deviceModelAttributes, deviceModelActions: deviceModelActions, deviceModelFormats: nil)
    }
    
    init(urn: String, name: String, description: String, deviceModelAttributes: [DeviceModelAttribute]?, deviceModelActions: [DeviceModelAction]?, deviceModelFormats: [DeviceModelFormat]?) {

        self.urn = urn
        self.name = name
        self.description = description
        
        if (deviceModelAttributes != nil) {
            for i in 0..<(deviceModelAttributes!.count) {
                if self.deviceModelAttributes[deviceModelAttributes![i].getName()!] == nil {
                    self.deviceModelAttributes[deviceModelAttributes![i].getName()!] = deviceModelAttributes![i]
                }
            }
        }
        if (deviceModelActions != nil) {
            for i in 0..<(deviceModelActions!.count) {
                if self.deviceModelActions[deviceModelActions![i].getName()] == nil {
                    self.deviceModelActions[deviceModelActions![i].getName()] = deviceModelActions![i]
                }
            }
        }
        if (deviceModelFormats != nil) {
            for i in 0..<(deviceModelFormats!.count) {
                if self.deviceModelFormats[deviceModelFormats![i].getURN()] == nil {
                    self.deviceModelFormats[deviceModelFormats![i].getURN()] = deviceModelFormats![i]
                }
            }
        }
    }
    
    func getURN() -> String {
        return self.urn
    }
    
    func getName() -> String {
        return self.name
    }
    
    func getDescription() -> String {
        return self.description
    }
    
    func getDeviceModelAttributes() -> [String : DeviceModelAttribute] {
        return self.deviceModelAttributes;
    }
    
    func getDeviceModelActions() -> [String : DeviceModelAction] {
        return self.deviceModelActions
    }
    
    func getDeviceModelFormats() -> [String : DeviceModelFormat] {
        return self.deviceModelFormats
    }
    
    func toString() -> String {
        var firstItem: Bool = true
        var str: String = "urn = " + "\t" + self.urn
        str = str + ",\n\tname = " + self.name
        str = str + ",\n\tdescription = " + self.description
        str = str + ",\n\tattributes = ["
        
        for attribute in Array(deviceModelAttributes.values) {
            if (!firstItem) {
                str = str + ","
            } else {
                firstItem = false;
            }
            
            str = str + "\n\t{"
            str = str + attribute.getName()!
            str = str + "}"
        }
        
        if (!firstItem) {
            str = str + "\n\t"
        }
        
        str = str + "],\n\tactions = ["
        
        firstItem = true;
        for action in Array(deviceModelActions.values) {
            if (!firstItem) {
                str = str + ","
            } else {
                firstItem = false;
            }
            
            str = str + "\n\t{"
            str = str + action.getName()
            str = str + "}"
        }
        
        if (!firstItem) {
            str = str + "\n\t"
        }
        
        str = str + "],\n\tformats = ["
        
        firstItem = true
        for format in Array(deviceModelFormats.values) {
            if (!firstItem) {
                str = str + ","
            } else {
                firstItem = false;
            }
            
            str = str + "\n\t{"
            str = str + format.getURN()
            str = str + "}"
        }
        
        if (!firstItem) {
            str = str + "\n\t"
        }
        
        str = str + "]"
        
        return str
    }
    
    // override Hashable
    var hashValue: Int {
        return self.urn.hashValue  // <--- This means only the urn are compared
    }
}

func ==(lhs: DeviceModelImpl, rhs: DeviceModelImpl) -> Bool {
    return lhs.hashValue == rhs.hashValue
}
