/*
 * Copyright (c) 2017, Oracle and/or its affiliates.  All rights reserved.
 *
 * This software is dual-licensed to you under the MIT License (MIT) and
 * the Universal Permissive License (UPL).  See the LICENSE file in the root
 * directory for license terms.  You may choose either license, or both.
 */

//package oracle.iot.client

/**
 * ExternalObject represents the value of a URI type in a data model.
 * The application is responsible for uploading/downloading the content referred
 * to by the URI.
 */
public class ExternalObject : Hashable {
    
    /// Returns a Boolean value indicating whether two values are equal.
    ///
    /// Equality is the inverse of inequality. For any values `a` and `b`,
    /// `a == b` implies that `a != b` is `false`.
    ///
    /// - Parameters:
    ///   - lhs: A value to compare.
    ///   - rhs: Another value to compare.
    public static func ==(lhs: ExternalObject, rhs: ExternalObject) -> Bool {
        return lhs.uri == rhs.uri
    }
    
    public var hashValue: Int {
        get {
            return 31 &+ uri.hashValue
        }
    }
    
    private let uri:String!

    /**
     * Get the URI value.
     *
     * - returns: the URI
     */
    public func getURI() -> String {
        return uri
    }

    /**
     * Create an `ExternalObject`.
     *
     * - parameter uri: The URI
     */
    public init(uri:String) {
        self.uri = uri
    }
}
