/*
 *  Copyright © 2016, 2018, Oracle and/or its affiliates. All rights reserved.
 *  This software is dual-licensed to you under the MIT License (MIT) and
 *  the Universal Permissive License (UPL). See the LICENSE file in the root
 *  directory for license terms. You may choose either license, or both.
 */

import Foundation
import os

// Hopefully at some point the os.log facility will provide for
// use of String instead of StaticString so that log entries can be
// localized

func log(type:OSLogType, message:String, args:CVarArg...) {
    // The real prototype doesn't support String only literal strings "FOO"
    //func os_log(_ message: StaticString, 
    //            dso: UnsafeRawPointer? = #dsohandle,
    //            log: OSLog = default, 
    //            type: OSLogType = default, _ args: CVarArg...)
    print(logMsg(type:type, message:message, args: args))
}

func stackTrace() -> String {
    var trace:String = ""
    Thread.callStackSymbols.forEach { sym in
        trace = trace + sym + "\n"
    }
    return trace
}

func logMsg(type:OSLogType,
            message:String, args:[CVarArg]! = nil) -> String {
    let logType:String = getLogType(type:type)
    return String(format:NSLocalizedString(message, comment:logType),
                  arguments:args)
}

fileprivate func getLogType(type:OSLogType) -> String {
    var logtype:String = "debug"
    switch (type) {
    case OSLogType.debug:
        logtype = "debug"
        break
    case OSLogType.info:
        logtype = "info"
        break
        
    case OSLogType.fault:
        logtype = "fault"
        break
    case OSLogType.error:
        logtype = "error"
        break
        /*
         case OSLogType.default:
         logtype = "default"
         break
         */
    default:
        logtype = "default"
        break
    }
    return logtype
}
