/*
 * Copyright (c) 2017, Oracle and/or its affiliates.  All rights reserved.
 *
 * This software is dual-licensed to you under the MIT License (MIT) and
 * the Universal Permissive License (UPL).  See the LICENSE file in the root
 * directory for license terms.  You may choose either license, or both.
 */

//com.oracle.iot.client.impl.util;

import Foundation

// TODO: Really want two hashable members
// but couldn't get it to work.
 /**
  * A convenience class to represent key-value pairs.
  */
public class Pair<T:Hashable,U:Any>: Hashable {
    /**
     * The key of this `Pair`.
     */
    private var key:T!

    /**
     * Value of this `Pair`.
     */
    private var value:U!

    /**
     * Gets the `key` for this pair.
     * - returns: `key` for this pair
     */
    public func getKey() -> T {
        return key
    }

    /**
     * Gets the `value` for this pair.
     * - returns: `value` for this pair
     */
    public func getValue() -> U {
        return value
    }

    /**
     * Creates a new pair
     *
     * - parameter key: The key for this pair
     *
     * - parameter value: The value to use for this pair
     */
    public init(key:T, value:U) {
        self.key = key
        self.value = value
    }

    /**
     * The `String` representation of this `Pair`.
     * The default key/value delimiter '=' is always used.
     *
     *  -returns: The `String` representation of this `Pair`
     */
    public func toString() -> String {
        return "\(String(describing: key)) = " +
            (value == nil ? "nil" : "\(value!)")
    }

    /* *
     * Generate a hash code for this `Pair`
     *
     * The hash code is calculated using both the key and
     * the value of the `Pair`
     *
     * - returns: hash code for this `Pair`
     */
    public var hashValue:Int {
        var hash:Int = 7
        //hash = 31 &* hash &+ key.hashValue
        // TODO: ObjectIndentifier may not be the way to go here.
        // It probably return a unique value unless value is the same instance
        //hash = 31 &* hash &+ String(describing: ObjectIdentifier(value)).hashValue
        
        hash = 31 &* hash &+ NSDictionary(dictionary:[key:value as Any]).hashValue
        return hash
    }

    /**
     * Test this `Pair` for equality with another `Pair`.
     *
     * Two `Pair`s are considered equal if and only if
     * both the `key`s and `value`s are equal.
     *
     * - parameter lhs: the `Pair` to test for
     *               equality with `rhs`
     *
     * - parameter rhs: the `Pair` to test for
     *               equality with `lhs`
     *
     * - returns: `true` if the given `rhs` is
     *           equal to `lhs` else `false`
     */
    //public static func ==<T:AnyHashable,U:Any>(lhs:Pair<T,U>, rhs:Pair<T,U>) -> Bool {
    public static func ==/*<T:Hashable,U:Any>*/(lhs:Pair<T,U>, rhs:Pair<T,U>) -> Bool {

        if lhs === rhs {
            return true
        }
        if lhs.key != rhs.key {
            return false
        }
        if lhs.value == nil && rhs.value == nil {
            return true
        }
        if let lhsvalue = lhs.value, let rhsvalue = rhs.value {
            // Hack to overcome a value that is not Hashable
            return NSDictionary(dictionary:[lhs.key : lhsvalue]).isEqual(to: [rhs.key : rhsvalue])
        } else {
            return false
        }
    }
}
