/*
 *  Copyright © 2016 Oracle and/or its affiliates. All rights reserved.
 *  This software is dual-licensed to you under the MIT License (MIT) and
 *  the Universal Permissive License (UPL). See the LICENSE file in the root
 *  directory for license terms. You may choose either license, or both.
 */

import Foundation

/**
 * RestApi provides the root path to the REST API version. Setting the
 * stored property `USE_WEBAPI` to `true` will
 * cause the code to use `/iot/webapi` instead of `/iot/api`.
 */
class RestApi {
    /**
     * Setting this property to `true` will
     * cause the code to use `/iot/webapi` instead of `/iot/api`.
     */
    static var USE_WEBAPI: Bool = false
    
    static let V1: RestApi = RestApi(version: "v1")
    static let V2: RestApi = RestApi(version: "v2")
    
    internal var reqRoot: String!
    internal var privateRoot: String!
    internal var webApi: Bool!
    
    init(version: String) {
        self.webApi = RestApi.USE_WEBAPI
        self.reqRoot = ((self.webApi == true) ?
            "/iot/webapi/" : "/iot/api/") + version
        self.privateRoot = ((self.webApi == true)  ?
            "/iot/privatewebapi/" : "/iot/privateapi/") + version
    }
    
    func getReqRoot() -> String {
        return self.reqRoot!
    }
    
    func getPrivateRoot() -> String {
        return self.privateRoot!
    }

    
    func isWebApi() -> Bool {
        return self.webApi!
    }
}
