/*
 * Copyright (c) 2017, Oracle and/or its affiliates.  All rights reserved.
 *
 * This software is dual-licensed to you under the MIT License (MIT) and 
 * the Universal Permissive License (UPL).  See the LICENSE file in the root
 * directory for license terms.  You may choose either license, or both.
 */

//package com.oracle.iot.client

import SwiftyJSON

/**
 * StorageAuthenticationResponse
 */
public class StorageAuthenticationResponse {

    private static let FIELD_STORAGE_CONTAINER_URL:String =
        "storageContainerUrl"
    private static let FIELD_AUTH_TOKEN:String = "authToken"

    private var storageContainerUrl:String?
    private var authToken:String?

    init() {
    }

    convenience init(storageContainerUrl:String?, authToken:String?) {
        self.init()
        self.storageContainerUrl = storageContainerUrl
        self.authToken = authToken
    }
    
    public func getAuthToken() -> String? {
        return authToken
    }

    public func setAuthToken(authToken:String) {
        self.authToken = authToken
    }
    
    public func getStorageContainerUrl() -> String? {
        return storageContainerUrl
    }

    
    public func setServicePath(storageContainedUrl:String) {
        self.storageContainerUrl = storageContainedUrl
    }
    

    /*
    public func toString() -> String {
        return "StorageAuthenticationResponse{" +
                "storageContainerUrl='" + storageContainerUrl ?? "" + '\'' +
                ", authToken=" + authToken ?? "" + '}'
    }
    */

    public static func fromJson(jsonObject:Foundation.Data) throws
            -> StorageAuthenticationResponse {

        let sar:StorageAuthenticationResponse = StorageAuthenticationResponse()
        let sarJson = try JSON(data:jsonObject)

        let storageContainerUrl:String? =
                sarJson.dictionaryValue[StorageAuthenticationResponse.FIELD_STORAGE_CONTAINER_URL]?.string
        if storageContainerUrl == nil {
            throw ClientError.network(NSLocalizedString(
                "json object does not contain \(StorageAuthenticationResponse.FIELD_STORAGE_CONTAINER_URL)",
                comment: ""))
        }
        sar.setServicePath(storageContainedUrl: storageContainerUrl!)
        let authToken = sarJson.dictionaryValue[StorageAuthenticationResponse.FIELD_AUTH_TOKEN]?.string
        if authToken == nil {
            throw ClientError.network(NSLocalizedString(
                "json object does not contain \(StorageAuthenticationResponse.FIELD_AUTH_TOKEN)",
                comment: ""))
        }
        sar.setAuthToken(authToken: authToken!)
        return sar
    }
    
    public func toJson() -> String {
        let json = String(format:
            "{\"\(StorageAuthenticationResponse.FIELD_STORAGE_CONTAINER_URL)\":\"%@\"," +
             "\"\(StorageAuthenticationResponse.FIELD_AUTH_TOKEN)\":\"%@\"}", storageContainerUrl!,
             authToken!)

        return json
    }
    
    public func clone() -> StorageAuthenticationResponse {
        return StorageAuthenticationResponse(storageContainerUrl: storageContainerUrl, authToken: authToken)
    }
}
