/*
 * Copyright (c) 2017, Oracle and/or its affiliates.  All rights reserved.
 *
 * This software is dual-licensed to you under the MIT License (MIT) and
 * the Universal Permissive License (UPL).  See the LICENSE file in the root
 * directory for license terms.  You may choose either license, or both.
 */

//package com.oracle.iot.client;
import Foundation

/**
 * The StorageConnection transfers content to the Storage Cloud Service.
 * StorageConnection is used by the client internally.
 */
public class StorageConnection: NSObject {
    
    /**
     * Synchronize content with the Storage Cloud Service. This method returns
     * immediately, and the callback is invoked when the network activity is 
     * complete.
     *
     * - paraemter storageObject: The `StorageObject` that indentifies the
     *                             content.
     * - parameter callback: `callback` is called when the network activity is
     *                       complete.
     * - throws: 
     *       `ClientError.network` if there is an exception raised by the
     *                             runtime or a failure reported by the
     *                             storage cloud
     *       `ClientError.security` if there is an exception establishing a
     *                              secure connection to the storage cloud
     *       `ClientError.argument` if the `StorageObject` does not return an
     *                              `InputStream` or `OutputStream`
     */
    func sync(storageObject: AdvancedStorageObject,
              callback: @escaping (ClientError?) -> ()) throws {
        fatalError("Subclasses need to implement the 'sync()' method")
    }
}

